package com.example.spaceaa;

import Exceptions.CertificateException;
import Exceptions.GetNonceException;
import Exceptions.KeyAggrementException;
import Exceptions.LastCommandException;
import Exceptions.MSESetATException;
import Exceptions.MapException;
import Exceptions.SelectException;
import Exceptions.TPcdException;
import Exceptions.TPiccException;
import android.nfc.NfcAdapter;
import android.nfc.Tag;
import android.nfc.tech.IsoDep;
import android.os.Bundle;
import android.app.Activity;
import android.app.PendingIntent;
import android.content.Context;
import android.content.Intent;
import android.view.Menu;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Toast;


/**
*
* @author Wojciech Wodo
*/
public class MainActivity extends Activity {

	private NfcAdapter adapter;
	private PendingIntent mPendingIntent;
	private SPACEAA protocol;
	private Button but;
	private IsoDep tag;
	private EditText pin;
	private EditText out;
	private CardCommunication card;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_main);

		but = ((Button) findViewById(R.id.button1));
		but.setOnClickListener(onClickListener);
		pin =  (EditText) findViewById(R.id.editText1);
		out =  (EditText) findViewById(R.id.editText2);
		
		adapter = NfcAdapter.getDefaultAdapter(this);
		if (!adapter.isEnabled()) {
			info("NFC is disabled.");
			return;
		}
		
		mPendingIntent = PendingIntent.getActivity(this, 0,
                new Intent(this, getClass()).addFlags(Intent.FLAG_ACTIVITY_SINGLE_TOP), 0);

	}

	@Override
	public void onResume(){
		super.onResume();
		adapter.enableForegroundDispatch(this, mPendingIntent, null, null);
	}
	
	@Override
	public void onPause(){
		super.onPause();
		adapter.disableForegroundDispatch(this);
	}
	
	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		// Inflate the menu; this adds items to the action bar if it is present.
		getMenuInflater().inflate(R.menu.main, menu);
		return true;
	}
	
	@Override
    public void onNewIntent(Intent intent) {
		Tag tagT = intent.getParcelableExtra(NfcAdapter.EXTRA_TAG);
		tag = IsoDep.get(tagT);
		if(tag==null){
			return;
		}
		card = new CardCommunicationNFC(tag);
		protocol = new SPACEAA(card);
		but.setEnabled(true);
    }
	
	public void info(String s){
		Context context = getApplicationContext();
		CharSequence text = s;
		int duration = Toast.LENGTH_LONG;
		
		Toast toast = Toast.makeText(context, text, duration);
		toast.show();
	}

	private OnClickListener onClickListener = new OnClickListener() {
	    @Override
	    public void onClick(final View v) {
				but.setEnabled(false);
	             switch(v.getId()){
	                 case R.id.button1:
	                	 info("Start SPACEAA");
	                	 startSPACEAA();
	                	 break;
	              }

	    }
	};
	
	private void startSPACEAA(){
		String result = "";
		if(pin.getText().toString().length()==0){
			info("PIN empty");
			return;
		}
		try {
			result = protocol.start(pin.getText().toString());
		} catch (MSESetATException e) {
			info("MSE Set AT Error");
		} catch (GetNonceException e) {
			info("Get Nonce Error");
		} catch (MapException e) {
			info("Mapping Error");
		} catch (KeyAggrementException e) {
			info("KeyAggrement Error");
		} catch (TPcdException e) {
			info("TPcd not accepted by card");
		} catch (TPiccException e) {
			info("TPicc from card not accepted");
		} catch (LastCommandException e) {
			info("Error in last command");
		} catch (CertificateException e) {
			info("Card Certificate Error");
		} catch (SelectException e) {
			info("Applet not selected");
		}
		if(result!=""){
			info("Communication channel created");
			out.setText(result);
		}
		card.disconnect();
	}
}
