package com.example.spaceaa;

import java.math.BigInteger;
import org.spongycastle.crypto.BlockCipher;
import org.spongycastle.crypto.Mac;
import org.spongycastle.crypto.digests.SHA1Digest;
import org.spongycastle.crypto.macs.CMac;
import org.spongycastle.crypto.modes.CBCBlockCipher;

/**
 *
 * @author Wojciech Wodo
 */
public class Util {
    
    public static byte[] concatenateByteArrays(byte[] a, byte[] b) {
        byte[] result = new byte[a.length + b.length]; 
        System.arraycopy(a, 0, result, 0, a.length); 
        System.arraycopy(b, 0, result, a.length, b.length); 
        return result;
    } 
    
    public static byte[] KDF_ENC(byte[] key){
        return KDF(key,1);
    }
    
    public static byte[] KDF_MAC(byte[] key){
        return KDF(key,2);
    }
    
    public static byte[] KDF_PI(String PIN){
        byte[] ret = new byte[PIN.length()];
        for(int i=0;i<PIN.length();i++){
            ret[i] = (byte)PIN.charAt(i);
        }
        return KDF(ret,3);
    }
    
    public static String byteToHexString(byte b){
        byte[] tab = {b};
        return byteHextoStringHex(tab);
        
    }
    
    public static String shorttoStringHex(short x) {
        String s="";
        byte[] ret=new byte[2];
        ret[1] = (byte)(x & 0xff);
        
        ret[0] = (byte)((x >> 8) & 0xff);
        
        s =(byteHextoStringHex(ret));
        
        return s;
    }
    
    public static String encodeLenToString(int lenn) {
        short len= (short)lenn;
        if (len <= 127) {
            return Util.byteToHexString((byte) (len));
        } else if (len > (short) 127 && len < (short) 256) {
            StringBuilder s = new StringBuilder("81 ");
            s = s.append(Util.byteToHexString((byte) (len & 0xFF)));
            return s.toString();
        } else {
            return "82 " + Util.shorttoStringHex(len);
        }
    }
    
    public static byte[] KDF(byte[] key, int counter){
        SHA1Digest h = new SHA1Digest();
        byte[] c = new byte[4];
        c[0]=(byte)0x00;
        c[1]=(byte)0x00;
        c[2]=(byte)0x00;
        c[3]=(byte)counter;
        byte[] in = Util.concatenateByteArrays(key, c);
        h.update(in, 0, in.length);
        byte[] ret = new byte[20];
        h.doFinal(ret,0);
        byte[] rett = new byte[16];
        for(int i=0;i<rett.length;i++){
            rett[i] = ret[i];
        }
        return rett;
    }
    
    public static String intToHexLen(int a){
        BigInteger b = new BigInteger(String.valueOf(a));
        String ret = b.toString(16);
        if(ret.length() % 2 != 0 ){
            ret = "0"+ret;
        }
        return ret;
    }
    
    
    public static String byteHextoStringHex(byte[] b) {
        String s = "";
        for (int i = 0; i < b.length; i++) {
            if ((byte)0x00 <= b[i] && b[i] <= (byte)0x0f) {
                s = s.concat("0").concat(Integer.toHexString(b[i])).concat(" ");
            } else {
                s = s.concat((Integer.toHexString(b[i]))).concat(" ");
            }
        }
        s = s.replace("ffffff", "");

        return s;
    }
    
    public static byte[] stringHexToByteHex(String s) {

        if (s.equals(new String(""))) {
            return new byte[0];
        }
        s = s.replaceAll(" ", "");

        //if ((s.length() % 2) != 0) {
        //    throw new IllegalArgumentException("String len is odd");
        //}

        byte[] data = new byte[s.length() / 2];
        for (int i = 0; i < s.length(); i += 2) {
            data[i / 2] = (byte) ((Character.digit(s.charAt(i), 16) << 4)
                    + Character.digit(s.charAt(i + 1), 16));
        }
        return data;

    }
    
    public static byte[] mac(byte[] message,byte[] key) {
        Mac mac= new CMac(new org.spongycastle.crypto.engines.AESEngine(),128);
        mac.init(new org.spongycastle.crypto.params.KeyParameter(key));
    	mac.update(message,0,message.length);
    	byte[] b=new byte[16];
    	mac.doFinal(b,0);
    	mac.reset();
        byte[] ret = new byte[8];
        for(int i=0;i<8;i++){
            ret[i] = b[i];
        }
        return ret;
    }
    
    public static byte[] dec(byte[] message,byte[] key){
        return cip(message,key,false);
    }
    public static byte[] enc(byte[] message,byte[] key){
        return cip(message,key,true);
    }
    
    public static byte[] cip(byte[] message,byte[] key,boolean mode) {
        CBCBlockCipher cipher = new CBCBlockCipher((BlockCipher)(new org.spongycastle.crypto.engines.AESEngine()));
        cipher.init(mode, new org.spongycastle.crypto.params.KeyParameter(key));
    	byte[] b=new byte[message.length];
        for(int i=0;i<message.length/16;i++){
            cipher.processBlock(message, i*16, b, i*16);
        }
    	cipher.reset();
        return b;
    }
    
    
}

